#HD44780
#--------------------------
#by Meysam Haji Mazdarani 
#pYBOARD 
#--------------------------

import pyb

class HD44780(object):

    PINS = ['Y1','Y2','Y3','Y4','Y5','Y6']
    PIN_NAMES = ['RS','E','D4','D5','D6','D7']
    pins = {}
    PIN_MODE = pyb.Pin.OUT_PP
    LCD_WIDTH = 16   
    LCD_CHR = True
    LCD_CMD = False

    LINES = {
        0: 0x80, 
        1: 0xC0, 
    }
    E_PULSE = 50
    E_DELAY = 50
    def init(self):

        for pin, pin_name in zip(self.PINS, self.PIN_NAMES):

            self.pins['LCD_'+pin_name] = pyb.Pin(pin, self.PIN_MODE)
        self.lcd_byte(0x33,self.LCD_CMD)
        self.lcd_byte(0x32,self.LCD_CMD)
        self.lcd_byte(0x28,self.LCD_CMD)
        self.lcd_byte(0x0C,self.LCD_CMD)
        self.lcd_byte(0x06,self.LCD_CMD)
        self.lcd_byte(0x01,self.LCD_CMD)

    def clear(self):
        self.lcd_byte(0x01,self.LCD_CMD)

    def set_line(self, line):
        self.lcd_byte(self.LINES[line], self.LCD_CMD)

    def set_string(self, message):
        m_length = len(message)
        if m_length < self.LCD_WIDTH:
            short = self.LCD_WIDTH - m_length
            blanks=str()
            for i in range(short):
                blanks+=' '
            message+=blanks
        for i in range(self.LCD_WIDTH):
            self.lcd_byte(ord(message[i]), self.LCD_CHR)

    def lcd_byte(self, bits, mode):


        self.pin_action('LCD_RS', mode) # RS
        self.pin_action('LCD_D4', False)
        self.pin_action('LCD_D5', False)
        self.pin_action('LCD_D6', False)
        self.pin_action('LCD_D7', False)
        if bits&0x10==0x10:
            self.pin_action('LCD_D4', True)
        if bits&0x20==0x20:
            self.pin_action('LCD_D5', True)
        if bits&0x40==0x40:
            self.pin_action('LCD_D6', True)
        if bits&0x80==0x80:
            self.pin_action('LCD_D7', True)

        self.udelay(self.E_DELAY)
        self.pin_action('LCD_E', True)
        self.udelay(self.E_PULSE)
        self.pin_action('LCD_E', False)
        self.udelay(self.E_DELAY)

        self.pin_action('LCD_D4', False)
        self.pin_action('LCD_D5', False)
        self.pin_action('LCD_D6', False)
        self.pin_action('LCD_D7', False)
        if bits&0x01==0x01:
            self.pin_action('LCD_D4', True)
        if bits&0x02==0x02:
            self.pin_action('LCD_D5', True)
        if bits&0x04==0x04:
            self.pin_action('LCD_D6', True)
        if bits&0x08==0x08:
            self.pin_action('LCD_D7', True)

        self.udelay(self.E_DELAY)
        self.pin_action('LCD_E', True)
        self.udelay(self.E_PULSE)
        self.pin_action('LCD_E', False)
        self.udelay(self.E_DELAY)

    def udelay(self, us):
        pyb.udelay(us)

    def pin_action(self, pin, high):
        if high:
            self.pins[pin].high()
        else:
            self.pins[pin].low()